package com.feedback.demo.service;

import com.feedback.demo.config.CustomUserDetail;
import com.feedback.demo.dto.ChangePasswordRequest;
import com.feedback.demo.dto.LoginResponse;
import com.feedback.demo.dto.SignupRequest;
import com.feedback.demo.entities.User;
import com.feedback.demo.exceptionhandle.ApiMissingException;
import com.feedback.demo.exceptionhandle.UsernameIsAlreadyTakenException;
import com.feedback.demo.repositories.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.io.IOException;

@Service
public class UserService {
    @Autowired
    private UserRepository userRepository;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Autowired
    private AuthenticationManager authenticationManager;

    @Autowired
    private JwtService jwtService;

    public User login(String username, String password) {
        Authentication authentication = authenticationManager
                .authenticate(new UsernamePasswordAuthenticationToken(username, password));
        SecurityContextHolder.getContext().setAuthentication(authentication);
        return getCurrentUser();
    }

    public User getCurrentUser() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (authentication instanceof AnonymousAuthenticationToken) {
            throw new ApiMissingException();
        }
        CustomUserDetail customUserDetail = (CustomUserDetail) authentication.getPrincipal();
        return customUserDetail.getUser();
    }

    public LoginResponse checkToken(String token) throws IOException {
        User user = jwtService.getUserFromToken(token);
        if (user != null) {
            String jwt = jwtService.generateToken(new CustomUserDetail(user));
            return new LoginResponse(user.getId(), user.getUsername(), jwt);
        } else {
            throw new RuntimeException("Token is valid");
        }
    }

    public LoginResponse getLoginResponse(String username, String password) {
        User user = login(username, password);
        String jwt = jwtService.generateToken(new CustomUserDetail(user));
        return new LoginResponse(user.getId(), user.getUsername(), jwt);
    }

    public LoginResponse changePassword(ChangePasswordRequest changePasswordRequest) {
        User user = getCurrentUser();
        if (passwordEncoder.matches(changePasswordRequest.getOldPass(), user.getPassword())) {
            user.setPassword(passwordEncoder.encode(changePasswordRequest.getNewPass()));
            userRepository.save(user);
            String jwt = jwtService.generateToken(new CustomUserDetail(user));
            return new LoginResponse(user.getId(), user.getUsername(), jwt);
        } else {
            throw new RuntimeException("Old password not match !");
        }
    }

    public LoginResponse signUp(SignupRequest signupRequest) {
        User userCheck = userRepository.findByUsername(signupRequest.getUsername());
        if (userCheck != null) {
            throw new UsernameIsAlreadyTakenException();
        }
        userRepository.save(
                User.builder()
                        .username(signupRequest.getUsername())
                        .password(
                                passwordEncoder.encode(signupRequest.getPassword())
                        )
                        .email(signupRequest.getEmail())
                        .phone(signupRequest.getPhone())
                        .domain(signupRequest.getDomain())
                        .build()
        );
        return getLoginResponse(signupRequest.getUsername(),signupRequest.getPassword());
    }
}
